* Output options: all can be turned on/off by adding or removing the NO;
* page number, date, centering, or page breaks, page length and width;
OPTIONS NOnumber NOdate NOcenter FormDlim=" " PageSize=MAX LineSize=MAX SPOOL;
* Eliminate SAS default titles and names of tables in output (TRACE ON to show);
TITLE; ODS TRACE OFF; *ODS LISTING;

***********************************************************************************;
*******     MACRO PROGRAMS TO AUTOMATE CALCULATIONS FOR THIS EXAMPLE        *******;
*******               NOTHING IN HERE NEEDS TO BE CHANGED                   *******;
***********************************************************************************;

* To use TotalR2 macro;
* DV =        Case-sensitive name of dependent variable (required);
* PredFewer = Name of OUTPM= data file of predicted outcomes for first model (required);
* PredMore =  Name of OUTPM= data file of predicted outcomes for second model (optional);
%MACRO TotalR2(DV=, Pred1=, Pred2=);
PROC CORR NOPRINT NOSIMPLE DATA=&Pred1. OUTP=Corr1; VAR pred &DV.; RUN;
DATA Corr1; LENGTH Name $30.; SET Corr1; Name="&Pred1.";
     IF Pred=. THEN Pred=0; PredCorr=Pred; TotalR2=PredCorr*PredCorr; 
     IF _NAME_="Pred" OR MISSING(_NAME_)=1 THEN DELETE; 
     KEEP Name PredCorr TotalR2; RUN;
%IF %LENGTH(&Pred2.)>0 %THEN %DO;
PROC CORR NOPRINT NOSIMPLE DATA=&Pred2. OUTP=Corr2; VAR pred &DV.; RUN;
DATA Corr2; LENGTH Name $30.; SET Corr2; Name="&Pred2."; 
     IF Pred=. THEN Pred=0; PredCorr=Pred; TotalR2=PredCorr*PredCorr; 
     IF _NAME_="Pred" OR MISSING(_NAME_)=1 THEN DELETE; 
     KEEP Name PredCorr TotalR2; RUN;
DATA CorrCompare; LENGTH Name $30.; SET Corr1 Corr2; 
     TotalR2Diff=TotalR2-LAG1(TotalR2);
     KEEP Name PredCorr TotalR2 TotalR2Diff; RUN; 
%END;
%IF %LENGTH(&Pred2.)>0 %THEN %LET ToPrint=CorrCompare; 
     %ELSE %LET ToPrint=Corr1;
TITLE8 "Total-R2 Per Model";
%IF %LENGTH(&Pred2.)>0 %THEN %DO;
TITLE9 "Change in Total-R2 for &Pred1. vs. &Pred2."; %END;
PROC PRINT NOOBS DATA=&ToPrint.; RUN; TITLE8; TITLE9;   
%MEND TotalR2;

* To use PseudoR2 macro;
* Ncov =     TOTAL # entries in covariance parameter estimates table;
* CovFewer = Name of ODS CovParms table for nested baseline model (required);
* CovMore1 = Name of ODS CovParms table for comparison model1 (required);
* CovMore2 = Name of ODS CovParms table for comparison model2 (optional);
%MACRO PseudoR2(NCov=, CovFewer=, CovMore1=, CovMore2=);
DATA &CovFewer.; LENGTH Name $30.; SET &CovFewer.; Name="&CovFewer."; RUN;
DATA &CovMore1.; LENGTH Name $30.; SET &CovMore1.; Name="&CovMore1."; RUN;
DATA CovCompare1; LENGTH Name $30.; SET &CovFewer. &CovMore1.; RUN;
DATA CovCompare1; SET CovCompare1; 
     PseudoR2=(LAG&Ncov.(Estimate)-Estimate)/LAG&Ncov.(Estimate); RUN;
%IF %LENGTH(&CovMore2.)>0 %THEN %DO; %LET lagchange=%EVAL(&Ncov.*2);
DATA &CovMore2.; LENGTH Name $30.; SET &CovMore2.; Name="&CovMore2."; RUN;
DATA CovCompare2; LENGTH Name $30.; SET &CovFewer. &CovMore2.; RUN;
DATA CovCompare2; SET CovCompare2; 
     PseudoR2=(LAG&Ncov.(Estimate)-Estimate)/LAG&Ncov.(Estimate); RUN;
DATA CovCompare; SET CovCompare1 CovCompare2;
     PseudoR2Change=PseudoR2-(LAG&lagchange(PseudoR2)); RUN;
%END; %IF %LENGTH(&CovMore2.)>0 %THEN %LET ToPrint=CovCompare; 
     %ELSE %LET ToPrint=CovCompare1;
DATA &ToPrint.; SET &ToPrint.; 
     IF CovParm IN("UN(2,1)","UN(3,1)","UN(3,2)","UN(4,1)","UN(4,2)","UN(4,3)",
                   "UN(5,1)","UN(5,2)","UN(5,3)","UN(5,4)",
                   "UN(6,1)","UN(6,2)","UN(6,3)","UN(6,4)","UN(6,5)") THEN DELETE; RUN;
TITLE8 "Pseudo-R2 Relative to &CovFewer.";
%IF %LENGTH(&CovMore2.)>0 %THEN %DO;
TITLE9 "Change in Pseudo-R2 for &CovMore1. vs. &CovMore2."; %END;
PROC PRINT NOOBS DATA=&ToPrint.; RUN; TITLE8; TITLE9;
%MEND PseudoR2;

***********************************************************************************;
*******           BEGIN DATA IMPORT AND PREPARATION FOR EXAMPLE 2           *******;
*******               CHANGE "filesave" TO YOUR FOLDER PATH                 *******;
***********************************************************************************;

* Define global variable for file location to be replaced in code below;
* \\Client\ precedes actual path when using UIowa Virtual Desktop;
%LET filesave=C:\Dropbox\23_PSQF6272\PSQF6272_Example2;

* Import trimmed example data excel data file from sheet "HSB4" into SAS;
PROC IMPORT DATAFILE="&filesave.\Example2_Data.xlsx" OUT=work.Example2 DBMS=XLSX REPLACE; 
     SHEET="HSB4"; GETNAMES=YES; RUN;

TITLE "SAS Descriptive Statistics for Example 2 Variables";
PROC MEANS DATA=work.Example2; VAR math size private; RUN; TITLE;

* Center and rescale school size (ref = 1000 students, per 100 students);
DATA work.Example2; SET work.Example2;
     size100 = (size-1000)/100; 
     * private is already 0/1; 
* Filter to only cases complete on all variables to be used below;
  IF NMISS(math,size,private)>0 THEN DELETE;
RUN;

* Compute mean cluster size;
PROC MEANS NOPRINT DATA=work.Example2; BY schoolID;
     VAR math; OUTPUT OUT=work.Schools MEAN(math)=SMmath; RUN;
TITLE "SAS Descriptive Statistics for Cluster Size and School Mean Math";
PROC MEANS DATA=work.Schools; VAR _FREQ_ SMmath; RUN; TITLE;

***********************************************************************************;
*******                    BEGIN MODELS FOR EXAMPLE 2                       *******;
***********************************************************************************;

* Open external file to save results to;
ODS RTF FILE="&filesave.\Example2_SAS_Output.rtf" 
        STYLE=HTMLBlue STARTPAGE=NO BODYTITLE;

TITLE "SAS Model 0: Single-Level Empty Means, NO Random Intercept";
PROC MIXED DATA=work.Example2 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID; 
     MODEL math = / SOLUTION DDFM=Satterthwaite; 
RUN; TITLE;

TITLE "SAS Model 1: Two-Level Empty Means, WITH Random Intercept";
PROC MIXED DATA=work.Example2 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID; 
     MODEL math = / SOLUTION DDFM=Satterthwaite; 
     RANDOM INTERCEPT / VCORR TYPE=UN SUBJECT=schoolID; * VCORR gives ICC;
     ODS OUTPUT CovParms=CovEmpty; * Save variances; 
RUN; TITLE;

TITLE "SAS Model 2: Add Main Effects of Level-2 School Size and Public vs Private";
PROC MIXED DATA=work.Example2 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID;         * OUTPM saves predicted outcomes to named dataset;
     MODEL math = size100 private / SOLUTION DDFM=Satterthwaite OUTPM=PredMain;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID; 
     ODS OUTPUT CovParms=CovMain; * Save variances; 
     CONTRAST "F-Test of Total R2" size100 1, private 1;
RUN; TITLE;
%PseudoR2(NCov=2, CovFewer=CovEmpty, CovMore1=CovMain);
%TotalR2(DV=math, Pred1=PredMain);

TITLE "SAS Model 3: Add Interaction of Level-2 School Size and Public vs Private";
PROC MIXED DATA=work.Example2 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID;             * OUTPM saves predicted outcomes to named dataset;
     MODEL math = size100|private / SOLUTION DDFM=Satterthwaite OUTPM=PredInteract;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID; 
     ODS OUTPUT CovParms=CovInteract; * Save variances; 
     ESTIMATE "Size slope for private schools"  size100 1 size100*private 1;
     CONTRAST "F-Test of Total R2"  size100 1, private 1, size100*private 1;
RUN; TITLE;
%PseudoR2(NCov=2, CovFewer=CovEmpty, CovMore1=CovMain, CovMore2=CovInteract);
%TotalR2(DV=math, Pred1=PredMain, Pred2=PredInteract);

* Close external file of saved results;
ODS RTF CLOSE;
