// Prevent "more" messages from appearing
   set more off
// Control line length
   set linesize 150

///////////////////////////////////////////////////////////////////////////////////////
////////               BEGIN DATA IMPORT AND PREPARATION FOR EXAMPLE 2          ///////
////////                  CHANGE "filesave" TO YOUR FOLDER PATH                 ///////
///////////////////////////////////////////////////////////////////////////////////////

// Define global variable for file location to be replaced in code below
// \\Client\ precedes path in Virtual Desktop outside H drive
   global filesave "C:\Dropbox\23_PSQF6272\PSQF6272_Example2"

// Open trimmed example excel data file from sheet "HSB4" and clear away any existing data
   clear // clear memory in case of open data
   import excel "$filesave\Example2_Data.xlsx", firstrow case(preserve) sheet("HSB4") clear 
   
display "STATA Descriptive Statistics for Example 2 Variables"
summarize math size private
     
// Center and re-scale school size
   gen size100 = (size-1000)/100
   // private is already 0/1

// Filter to only cases complete on all variables to be used below
   egen nmiss=rowmiss(math size private)
   drop if nmiss>0

// Open external file to save results to
   log using $filesave\Example2_STATA_Output.log, replace
   
   
///////////////////////////////////////////////////////////////////////////////////////
////////                     BEGIN MODELS FOR EXAMPLE 2                          //////
///////////////////////////////////////////////////////////////////////////////////////

display "STATA Model 0: Single-Level Empty Means, NO Random Intercept"
mixed math , || schoolID: , noconstant reml dfmethod(residual) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  
estat ic, n(160)                // AIC and BIC for # level-2 units


display "STATA Model 1: Two-Level Empty Means, WITH Random Intercept"
mixed math , || schoolID: , reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  
estat ic, n(160)                // AIC and BIC for # level-2 units
estat icc                       // Intraclass correlation
display "ICC2 = " 8.614081/(8.614081+(39.14832/45))
          
          
display "STATA Model 2: Add Main Effects of Level-2 School Size and Public vs Private"
mixed math c.size100 c.private, || schoolID: , reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  
estat ic, n(160)                // AIC and BIC for # level-2 units         
predict predmain                // Save fixed-effect predicted outcomes
corr math predmain              // Get total r to make R2
display "Total-R2 = " r(rho)^2  // Print total R2 relative to empty model       


display "STATA Model 3: Add Interaction of Level-2 School Size and Public vs Private"
mixed math c.size100##c.private , || schoolID: , reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  
estat ic, n(160)                // AIC and BIC for # level-2 units
lincom c.size100*1 + c.size100#c.private // Size slope for private schools      
predict predinteract            // Save fixed-effect predicted outcomes
corr math predinteract          // Get total r to make R2
display "Total-R2 = " r(rho)^2  // Print total R2 relative to empty model       


// Close external file of saved results
   log close

